/******************************************************************************
 * FILE:
 *    uc_spc574k72_core_regs_init.S
 *
 * UC:
 *    SPC574K72
 *
 * DESCRIPTION:
 *    Lock-step Core specific registers' initialization.
 *
 * FILE HISTORY:
 *    V1.0    29.01.2018     RO
 *       > Initial version based on crt0_spc58, dedicated to target uC
 *         derivative
 * 
 *	  V2.0    30.01.2018     MF
 *       > file update for SPC574K72
 *
 ******************************************************************************
 * Copyright (C) 2015-2018 HighTec EDV-Systeme GmbH.  All rights reserved.
 * This is proprietary software. Any use of the software requires a written
 * license agreement with HighTec EDV-Systeme GmbH. Please contact for
 * detailed license information: info@hightec-rt.com.
 *****************************************************************************/


/* ============================================================================
 * SYMBOLS & DEFINES & MACROS
 * ==========================================================================*/

/* Core identification values (PVR.type & PVR.version bitfields)
 * values taken from e200 Cores' manuals */
#define CORE_MAIN_0    0
#define CORE_IOP       2

/* Exported symbols
 * IVOR00 : base address of the Exception table */
.global ls_regs_init
.type ls_regs_init STT_FUNC


/* ============================================================================
 * LOCK-STEP CORE SELECTION
 * Decide which Core register set to initialize, if any, based on CoreId read
 * in the runtime
 * ==========================================================================*/

/* crt0 shared code section */
	.section .crt0, "avx"

ls_regs_init:
	mfspr     r3, 286                   /* read CoreID SFR register */

	e_cmpl16i r3, CORE_IOP
	beq       _z225Bn3_reg_init

	e_cmpl16i r3, CORE_MAIN_0
	beq       _z410Dn3_reg_init

ls_regs_init_end:
	se_blr                              /* return to caller */


/* ============================================================================
 * LOCK-STEP CORE REGISTER INIT
 * Certain Core registers running in Lockstep needs to be initialized by write
 * operation before they are read back, which might cause an Lockstep error,
 * if their value after Power-On is different.
 * e200z4 Core RM Chapter 2.6 Reset settings
 * ==========================================================================*/

_z410Dn3_reg_init:
	mtcrf 	0xFF, r0	//CR
	mtspr 	58,	  r0   	//CSRR0
	mtspr 	59,   r0   	//CSRR1
	mtspr 	9,    r0   	//CTR
	mtspr 	600,  r0  	//DDEAR
	mtspr 	61,   r0   	//DEAR
	mtspr 	574,  r0  	//DSRR0
	mtspr 	575,  r0  	//DSRR1
	mtspr 	601,  r0  	//DVC1
	mtspr 	601,  r0	//DVC1U
	mtspr 	318,  r0	//DVC2
	mtspr 	318,  r0  	//DVC2U
	mtspr 	63,   r0 	//IVPR
	mtspr 	624,  r0  	//MAS0
	mtspr 	625,  r0  	//MAS1
	mtspr 	626,  r0  	//MAS2
	mtspr 	627,  r0  	//MAS3
    mtspr 	573,  r0  	//MCAR
	mtspr 	570,  r0  	//MCSRR0
	mtspr 	571,  r0  	//MCSRR1
	mtspr 	272,  r0  	//SPRG0
	mtspr 	273,  r0  	//SPRG1
	mtspr 	274,  r0  	//SPRG2
	mtspr 	275,  r0  	//SPRG3
	mtspr 	26,   r0   	//SRR0
	mtspr 	27,   r0   	//SRR1
	mtspr 	256,  r0   	//USPRG0
	se_blr

_z225Bn3_reg_init:

se_blr

